#!/bin/bash

# TODO: support run by java main class

script_dir() {
    # Default is current directory
    local dir=$(dirname "$0")
    local full_dir=$(cd "${dir}" && pwd)
    echo ${full_dir}
}

load_env() {

    # TODO: run run-env.sh if there is

    JAVA_APP_NAME="${JAVA_APP_NAME:-app}"
    export JAVA_APP_NAME

    JAVA_APP_DIR="${JAVA_APP_DIR:-$(script_dir)}"
    export JAVA_APP_DIR

    JAVA_LIB_DIR="${JAVA_LIB_DIR:-$JAVA_APP_DIR}"
    export JAVA_LIB_DIR

    SERVER_JVM_OPTS="${SERVER_JVM_OPTS:-}"
    export SERVER_JVM_OPTS

}

init_java_major_version() {
    # Initialize JAVA_MAJOR_VERSION variable if missing
    if [ -z "${JAVA_MAJOR_VERSION:-}" ]; then
        local full_version=""

        # Parse JAVA_VERSION variable available in containers
        if [ -n "${JAVA_VERSION:-}" ]; then
            full_version="$JAVA_VERSION"
        elif [ -n "${JAVA_HOME:-}" ] && [ -r "${JAVA_HOME}/release" ]; then
            full_version="$(grep -e '^JAVA_VERSION=' ${JAVA_HOME}/release | sed -e 's/.*\"\([0-9.]\{1,\}\).*/\1/')"
        else
            full_version=$(java -version 2>&1 | head -1 | sed -e 's/.*\"\([0-9.]\{1,\}\).*/\1/')
        fi
        export JAVA_MAJOR_VERSION=$(echo $full_version | sed -e 's/[^0-9]*\(1\.\)\{0,1\}\([0-9]\{1,\}\).*/\2/')
    fi
}

#gc_options() {
#    # no-op if already configured
#    if echo "${SERVER_JVM_OPTS:-}" | grep -q -- "-XX:.*Use.*GC"; then
#        return
#    fi
#}
#
#diagnostics_options() {
#  if [ -n "${JAVA_DIAGNOSTICS:-}" ]; then
#    if [ "${JAVA_MAJOR_VERSION:-0}" -ge "11" ]; then
#      echo "-XX:NativeMemoryTracking=summary -Xlog:gc*:stdout:time -XX:+UnlockDiagnosticVMOptions"
#    else
#      echo "-XX:NativeMemoryTracking=summary -XX:+PrintGC -Xlog:gc*   -XX:+UnlockDiagnosticVMOptions"
#    fi
#  fi
#}
#
#memory_options() {
#
#}
#
#calc_init_mem() {
#
#}
#
#calc_max_mem() {
#
#}

debug_options() {
    if [ -n "${JAVA_ENABLE_DEBUG:-}" ] || [ -n "${JAVA_DEBUG_ENABLE:-}" ] || [ -n "${JAVA_DEBUG:-}" ]; then
        local debug_port="${JAVA_DEBUG_PORT:-5005}"
        local suspend_mode="n"
        if [ -n "${JAVA_DEBUG_SUSPEND:-}" ]; then
            if ! echo "${JAVA_DEBUG_SUSPEND}" | grep -q -e '^\(false\|n\|no\|0\)$'; then
                suspend_mode="y"
            fi
        fi

        local address_prefix=""
        if [ "${JAVA_MAJOR_VERSION:-0}" -ge "9" ]; then
            address_prefix="*:"
        fi
        echo "-agentlib:jdwp=transport=dt_socket,server=y,suspend=${suspend_mode},address=${address_prefix}${debug_port}"
    fi
}

# Set process name if possible
exec_args() {
    EXEC_ARGS=""
    if [ -n "${JAVA_APP_NAME:-}" ]; then
        # Not all shells support the 'exec -a newname' syntax..
        if $(exec -a test true 2>/dev/null); then
            echo "-a '${JAVA_APP_NAME}'"
        fi
    fi
}

# Combine all java options
java_options() {
    # Normalize spaces with awk (i.e. trim and elimate double spaces)
    echo "${SERVER_JVM_OPTS:-} $(debug_options)" | awk '$1=$1'
}

run_java() {
    load_env

    cd ${JAVA_APP_DIR}

    if [ -z "${JAVA_APP_JAR:-}" ]; then
        echo "JAVA_APP_JAR needs to be given"
        exit 1
    fi

    exec $(exec_args) java $(java_options) -jar "${JAVA_APP_JAR}" "$@"
}

init_java_major_version

run_java "$@"
